import os,sys,shutil,time
import bagit
import zipfile
import xml.etree.ElementTree as ET
import ipywidgets as ui
from ipywidgets import Layout
import IPython
from IPython import display
from IPython.display import Javascript
from hublib.ui import FileUpload
from hublib.ui import Modal
from FolderSelector import FolderSelector
from ipywidgets import FloatProgress
from iData import iData

class BagImporter():

    def __init__(self):
        sys.path.insert(0, os.path.abspath('..'))
        #get iData client
        self.idata_client = iData()
        #first the various UI elements
        self.folderselect = FolderSelector('/srv/irods')
        self.folderselect.disabled = True
        self.extract_button = ui.Button(description='Extract here',disabled=True)
        self.extract_button.on_click(self.extract_bag)
        self.baguploader = FileUpload("Select BagIT bag for processing", 
                                      "Select BagIT bag for processing to extract contents and file metadata",
                                       dir='tmpdir',
                                       cb=self.done_cb,
                                       maxsize='200M')
        self.control_ui = ui.HBox([self.extract_button])
        self.dest_label = ui.HTML(value="<b>Select project folder to extract this bag into...<b>")
        self.dest_ui = ui.VBox([self.dest_label,self.folderselect.accord,self.control_ui])
        self.status_ui = ui.Output()
        self.gui = ui.VBox([self.baguploader.w,self.dest_ui,self.status_ui])
        
        #next the metadata fields and namespaces
        self.ns = {'xmlns:dcterms':"http://purl.org/dc/terms/",
                  'xmlns:rdf':"http://www.w3.org/1999/02/22-rdf-syntax-ns#",
                  'xmlns:dc':"http://purl.org/dc/elements/1.1/",
                  'xmlns:terms':"https://mygeohub.org/terms/"}

        self.core_fields = ('description','title','subject','contributor','publisher',
                           'date','identifier','format','type','source','creator')

        self.derived_core_fields = ('identifier','contributor','publisher','format','type','date','coverage')

        self.derived_term_fields = ('language','lonmin','lonmax','latmin','latmax',
                                   'owner','owner_type','xsize','ysize')

        self.inner_tags = {'description':['dcterms:abstract','rdf:Description'],
                          'contributor':['rdf:Description'],
                          'publisher':['rdf:Description'],
                          'date':['rdf:value'],
                          'identifier':['rdf:Description'],
                          'creator':['rdf:Description']}
        
        #the bag's extracted directory
        self.bagdir = ''
        #destination for a bag's contents
        self.destdir = ''
        #number of files included
        self.numfiles = 0
        
    #returns the basename of a filename argument, i.e. without the extension
    def process_filename(self,filename):
        base_name = os.path.basename(filename)
        return os.path.splitext(base_name)[0]

    #custom modal dialog
    def modal_dialog(self,title,body):
        data = """
           require(
               ["base/js/dialog"],
                function(dialog) {
                   dialog.modal({
                       title: '%s',
                       body: '%s',
                       sanitize: false,
                       buttons: {
                           'Close': {}
                       }
               });
           })
           """ % (title,body)
        display.display(Javascript(data=data))

    def throw_error(self,message):
        self.modal_dialog('ERROR',message)
        
    def throw_success(self,message):
        self.modal_dialog('SUCCESS',message)

    def done_cb(self,dummy,name):
        #clear any previous statuses
        with self.status_ui:
            IPython.display.clear_output()
        #check if it is a zip file first
        if zipfile.is_zipfile(name[0]):
            #create a directory to unzip this file to
            #check if it is a valid BagIT bag
            self.make_bag_directory(name[0])
            bag_zip = zipfile.ZipFile(name[0],'r')
            bag_zip.extractall(path=self.dirname)
            bag = bagit.Bag(self.dirname)
            if not bag.is_valid():
                self.throw_error("ERROR: this is not a valid BagIT bag.")
            else:
                #enable selection of a destination project folder
                self.folderselect.disabled = False
                self.extract_button.disabled = False
                #find number of files
                content_dir = '%s/data/contents' % self.dirname
                count = 0
                for listing in os.listdir(content_dir):
                    count += 1
                self.numfiles = count
        else:
            self.throw_error("ERROR: Unrecognized format; only zipfiles of BagIT bags are supported.")
        
        # reset clears and re-enables the widget for more uploads
        # You may want to do this somewhere else, depending on your GUI
        self.baguploader.reset()
        
        return
    
    def make_bag_directory(self,filename):
        file_basename = self.process_filename(filename)
        self.dirname = 'tmpdir/%s' % file_basename
        if not os.path.exists(self.dirname):
            os.makedirs(self.dirname)
        return

    def is_project_folder(self,fold):
        if fold.startswith('/srv/irods/'):
            return True
        else:
            return False
        
    def copy_bag_contents(self):
        #listing of files from content directory
        content_dir = '%s/data/contents' % self.dirname
        try:
            for listing in os.listdir(content_dir):
                self.progbar.description = 'Copying file %s' % listing
                filename = '%s/%s' % (content_dir,listing)
                #copy to destination project folder
                if os.path.isfile(filename):
                    shutil.copy(filename,self.destdir)
                    self.progbar.value += 10
                    time.sleep(.2)
            return True
        except:
            self.throw_error("ERROR: Could not extract one or more files from bag into project...")
            return False
                
    
    def process_serialized_metadata(self):
        meta_filename = '%s/data/metadata.xml' % self.dirname
        tree = ET.parse(meta_filename)
        root = tree.getroot()
        #these are the files contained in here
        #each metadata block will need to identify the file by the id attribute
        for child in root:
            #dictionary of metadata fields to set
            meta_kvdict = dict()
            #get filename
            id_key = '{%s}ID' % self.ns['xmlns:rdf']
            filepath = '%s/%s' % (self.destdir,child.get(id_key))
            self.progbar.description = 'Setting metadata on %s' % child.get(id_key)
            for field in self.core_fields:
                #if this is a derived field, the microservice will regenerate it
                if field not in self.derived_core_fields:
                    field_tag = '{%s}%s' % (self.ns['xmlns:dc'],field)
                    if field in self.inner_tags:
                        inner_tag = ''
                        for inner_field in self.inner_tags[field]:
                            data = inner_field.split(':')
                            data_ns = 'xmlns:%s' % data[0]
                            data_tag = data[1]
                            if len(inner_tag) > 0:
                                inner_tag = '{%s}%s/%s' % (self.ns[data_ns],data_tag,inner_tag)
                            else:
                                inner_tag = '{%s}%s' % (self.ns[data_ns],data_tag)
                        field_tag = '%s/%s' % (field_tag,inner_tag)
                        xml_ele = child.find(field_tag)
                        if xml_ele is not None:
                            meta_kvdict[field] = xml_ele.text
                    else:
                        xml_ele = child.find(field_tag)
                        if xml_ele is not None:
                            meta_kvdict[field] = xml_ele.text
            for children in child:
                terms_ns = '{%s}' % self.ns['xmlns:terms']
                flag = False
                for term_field in self.derived_term_fields:
                    term_tag = '{%s}%s' % (self.ns['xmlns:terms'],term_field)
                    if children.tag.startswith(terms_ns):
                        if term_tag == children.tag:
                            flag = True
                            break
                    else:
                        flag = True
                if not flag:
                    key = children.tag[len(terms_ns):]
                    meta_kvdict[key] = children[0].text
            #now invoke the metadata setter
            self.idata_client.set_idata_file_metadata(filepath,meta_kvdict)
            self.progbar.value += 10
            time.sleep(.2)
        return True
            
    def extract_bag(self,change):
        self.destdir = self.folderselect.value
        #if this is not a project folder, disallow extraction
        if not self.is_project_folder(self.destdir):
            self.throw_error("ERROR: Select a folder under /srv/irods as the destination...")
        else:
            #fix up the UI to prevent repeated requests to extract
            self.folderselect.disabled = True
            self.folderselect.accord.selected_index = None
            self.extract_button.disabled = True
            #create progress bar
            maxval = self.numfiles * 20
            self.progbar = ui.IntProgress(min=0,max=maxval,
                                          style={'description_width': 'initial'},
                                          layout=ui.Layout(width='95%'))
            with self.status_ui:
                display.display(self.progbar)
            #copy the bag contents
            res = self.copy_bag_contents()
            if res:
                #extract metadata from RDF and set any non-automatically derived metadata
                res = self.process_serialized_metadata()
                if res:
                    self.progbar.description = 'Bag extracted successfully...'
                    self.progbar.bar_style = 'success'
                else:
                    self.progbar.description = 'Error extracting bag...'
                    self.progbar.bar_style = 'danger'
