# ----------------------------------------------------------------------
# A widget that allows you to choose a single server-side directory.
# based on Rob Campbell's file selector
# ======================================================================
#  AUTHOR: Rajesh Kalyanam, Purdue University
#  Copyright (c) 2004-2017  HUBzero Foundation, LLC
#  See LICENSE file for details.
# ======================================================================
import os
import ipywidgets as ui
from IPython.display import display

class FolderSelector():

    def __init__(self,start_dir):
        self.file        = None 
        self.value       = start_dir
        self.selected_files = []
        self.select      = ui.Select(options=['init'],value=None,rows=10,description='') 
        self.accord      = ui.Accordion(children=[self.select]) 
        
        self.accord.selected_index = None # Start closed (showing path only)
        self.refresh(self.value)
        self.select.observe(self.on_update,'value')
            
    def on_update(self,change):
        if not self.select.disabled:
            if change['new'] is not None:
                self.refresh(change['new'])

    def refresh(self,item):
        path = os.path.abspath(os.path.join(self.value,item))
        
        if os.path.isfile(path):
            self.select.value = None

        else: # os.path.isdir(path)
            self.file = None 
            self.value  = path
            
            # Build list of files and dirs
            keys = ['[..]']; 
            for item in os.listdir(path):
                if item[0] == '.':
                    continue
                elif os.path.isdir(os.path.join(path,item)):
                    keys.append('['+item+']'); 
                else:
                    keys.append(item); 

            # Sort and create list of output values
            keys.sort(key=str.lower)
            vals = []
            for k in keys:
                if k[0] == '[':
                    vals.append(k[1:-1]) # strip off brackets
                else:
                    vals.append(k)

            # Update widget
            self.select.disabled = True
            self.accord.set_title(0,path)  
            self.select.options = list(zip(keys,vals)) 
            self.select.disabled = False
            self.select.value = None

    def _ipython_display_(self):
        self.accord._ipython_display_()
        
    @property
    def disabled(self):
        return self.select.disabled

    @disabled.setter
    def disabled(self, newval):
        self.select.disabled = newval

    @property
    def visible(self):
        return self.accord.layout.visibility

    @visible.setter
    def visible(self, newval):
        if newval:
            self.accord.layout.visibility = 'visible'
            return
        self.accord.layout.visibility = 'hidden'

