import xml.etree.ElementTree as ET
import os
from datetime import datetime
from datetime import date
from XMLUtils import XMLUtils

class MetadataSerializer():
    
    def __init__(self,dcmi_metadata,identifier):
        self.dcmi_metadata = dict()
        for k,v in dcmi_metadata.items():
            self.dcmi_metadata[k] = v
        self.rdf_metadata_tree = None
        #filename in case we don't have that info in the metadata
        self.identifier = identifier
        self.core_fields = ('description','title','subject','contributor','publisher',
                       'date','identifier','format','type','source','creator')

        self.inner_tags = {'description':['dcterms:abstract','rdf:Description'],
                      'contributor':['rdf:Description'],
                      'publisher':['rdf:Description'],
                      'date':['rdf:value'],
                      'identifier':['rdf:Description'],
                      'creator':['rdf:Description']}

        self.geo_fields = ('projection','northlimit','southlimit','eastlimit','westlimit')
        
        self.URL = 'https://mygeohub.org'

    #just a basic element with no children
    #def create_xml_element_basic(self,tag,value):
    #    res = ET.Element(tag)
    #    res.text = value
    #    return res

    #create a xml element with one or more children
    #assumes element has no value or attributes; just the subelements
    #def create_xml_element(self,tag,children):
    #    res = ET.Element(tag)
    #    res.extend(children)
    #    return res
    
    #top-level function
    def create_rdf_metadata_xml(self):
        xmlutils = XMLUtils()
        xml_elements = []
        #first create elements for core fields
        for core_field in self.core_fields:
            xml_ele = None
            if core_field in self.dcmi_metadata:
                value = self.dcmi_metadata[core_field]
                core_field_tag = 'dc:%s' % core_field
                #check if we need to created nested elements
                if core_field in self.inner_tags:
                    innermost = True
                    child_ele = None
                    #recursively build the child element child_ele
                    for inner_field in self.inner_tags[core_field]:
                        if innermost:
                            child_ele = xmlutils.create_xml_element_basic(inner_field,value)
                            innermost = False
                        else:
                            child_ele = xmlutils.create_xml_element(inner_field,[child_ele])
                    #top-level element for the core field
                    xml_ele = xmlutils.create_xml_element(core_field_tag,[child_ele])
                else:
                    #top-level element for the core field
                    xml_ele = xmlutils.create_xml_element_basic(core_field_tag,value)
                if xml_ele is not None:
                    xml_elements.append(xml_ele)
                
        #now process the geospatial fields if any
        #will represent as single xml element with string of field=value as content
        geo_value = None
        for geo_field in self.geo_fields:
            if geo_field in self.dcmi_metadata:
                if geo_value is None:
                    geo_value = '%s=%s' % (geo_field,self.dcmi_metadata[geo_field])
                else:
                    geo_value = '%s;%s=%s' % (geo_value,geo_field,self.dcmi_metadata[geo_field])
        #now create the xml elements to hold geospatial coverage
        #again recursively
        if geo_value is not None:
            xml_ele = xmlutils.create_xml_element_basic('rdf:value',geo_value)
            xml_ele = xmlutils.create_xml_element('dcterms:box',[xml_ele])
            xml_ele = xmlutils.create_xml_element('dc:coverage',[xml_ele])
            xml_elements.append(xml_ele)
       
        #now process the remaining non-DCMI fields
        for meta_field in self.dcmi_metadata:
            if meta_field not in self.core_fields and meta_field not in self.geo_fields:
                value_ele = xmlutils.create_xml_element_basic('rdf:Description',self.dcmi_metadata[meta_field])
                xml_ele_tag = 'terms:%s' % meta_field
                xml_ele = xmlutils.create_xml_element(xml_ele_tag,[value_ele])
                xml_elements.append(xml_ele)
                
        #now create top-level elements
        #xml_ele = self.create_xml_element('rdf:Description',xml_elements)
        #xml_ele.set('rdf:about',self.URL)
        #res = self.create_xml_element('rdf:RDF',[xml_ele])
        #res.set('xmlns:dcterms',"http://purl.org/dc/terms/")
        #res.set('xmlns:rdf',"http://www.w3.org/1999/02/22-rdf-syntax-ns#")
        #res.set('xmlns:dc',"http://purl.org/dc/elements/1.1/")
        #res.set('xmlns:terms',"https://mygeohub.org/terms/")
        res = xmlutils.create_xml_element('rdf:Description',xml_elements)
        if 'identifier' in self.dcmi_metadata:
            res.set('rdf:ID',os.path.basename(self.dcmi_metadata['identifier']))
        else:
            res.set('rdf:ID',self.identifier)
        res.set('rdf:about',self.URL)
        etree = ET.ElementTree(res)
        self.rdf_metadata_tree = etree
        
        return

