import xml.etree.ElementTree as ET
import os
import pandas as pd
import ipywidgets as ui
from ipywidgets import Layout
import IPython
from IPython import display
from IPython.display import Javascript
from hublib.ui import Download
import random
import shutil
import bagit
from bs4 import BeautifulSoup
from MultiFileSelector import MultiFileSelector
from iData import iData
from MetadataSerializer import MetadataSerializer
from XMLUtils import XMLUtils

class iData_Packaging():
    
    def __init__(self):
        #first some regular fields
        #now the UI
        self.file_selector = MultiFileSelector('/srv/irods')
        self.file_selector.accord.observe(self.review_files,names='selected_index')
        self.items = []
        self.out = ui.Output()
        self.gui = ui.VBox([self.file_selector.accord,self.out])
        self.remove_button = ui.Button(description='Remove Selected Files')
        self.remove_button.layout.width = '200px'
        self.remove_button.on_click(self.remove_files)
        self.package_button = ui.Button(description='Package These Files')
        self.package_button.layout.width = '200px'
        self.package_button.on_click(self.package_files)
        self.bag_path = None
        self.bag_filepath = None
        #will provide checkboxes to select files to remove
        #what about packaging a whole directory (recursive)? with some limit on # of files you can include
        #so that Jupyter doesn't truncate long strings when display a DataFrame
        pd.set_option('display.max_colwidth', -1)
        
    #custom modal dialog
    def modal_dialog(self,title,body):
        data = """
            require(
                ["base/js/dialog"],
                function(dialog) {
                    dialog.modal({
                        title: '%s',
                        body: '%s',
                        sanitize: false,
                        buttons: {
                            'Close': {}
                        }
                });
            })
            """ % (title,body)
        display.display(Javascript(data=data,css='modal.css'))

    #create a directory to store the bag before zipping it up
    def create_directory(self,directory):
        try:
            os.stat(directory)
        except:
            try:
                os.mkdir(directory)
                return True
            except:
                return False
            
    #create a RDF XML file to store the metadata of the bag's files
    def make_serialized_meta_XML(self,meta_filename,meta_records):
        xmlutils = XMLUtils()
        rdf_file = open(meta_filename,'w')
        #create root element with one subelement for each file's metadata
        tree_ele = xmlutils.create_xml_element('rdf:RDF',meta_records)
        tree_ele.set('xmlns:dcterms',"http://purl.org/dc/terms/")
        tree_ele.set('xmlns:rdf',"http://www.w3.org/1999/02/22-rdf-syntax-ns#")
        tree_ele.set('xmlns:dc',"http://purl.org/dc/elements/1.1/")
        tree_ele.set('xmlns:terms',"https://mygeohub.org/terms/")
        tree = ET.ElementTree(tree_ele)
        root = tree.getroot()
        xmlstr = BeautifulSoup(ET.tostring(root), "xml")
        rdf_file.write(str(xmlstr))
        #rdf_file.write(xmlstr.prettify())
        #xmlstr = ET.tostring(root)
        #rdf_file.write('%s' % xmlstr)
        rdf_file.close()
        
        
    #package files as a BagIT bag
    def package_files(self,obj_ignored):
        files_to_package = self.file_selector.selected_files

        #create bag in session directory and then zip up
        self.bag_path = 'bag_%d' % random.randint(1,100)
        #create the directory to hold the bag's contents
        if self.create_directory(self.bag_path):
            #create sub folders for data and contents
            data_path = '%s/contents' % self.bag_path
            self.create_directory(data_path)
            rdf_filepath = '%s/metadata.xml' % self.bag_path
            idata_client = iData()
            rdf_resources = []
            for filepath in files_to_package:
                #first copy file to bag directory
                shutil.copy2(filepath,data_path)
                #returned metadata is a Python dictionary
                dcmi_metadata = idata_client.get_idata_file_metadata(filepath)
                meta_serializer = MetadataSerializer(dcmi_metadata,os.path.basename(filepath))
                meta_serializer.create_rdf_metadata_xml()
                #save the RDF metadata document for each file be packaged to the metadata text file
                res = meta_serializer.rdf_metadata_tree
                rdf_resources.append(res.getroot())
            self.make_serialized_meta_XML(rdf_filepath,rdf_resources)
            #create bagit bag
            bagit.make_bag(self.bag_path)
            #create zip of bag
            shutil.make_archive(self.bag_path,'zip',self.bag_path)
            self.bag_filepath = '%s.zip' % self.bag_path
            #delete bag directory
            shutil.rmtree(self.bag_path)
            
            download_bag = Download(self.bag_filepath, label='BagIT bag', icon='download', tooltip='DOWNLOAD as BagIT bag', width='500px')
            self.download_button = download_bag.w
            #disable buttons to package and remove files
            self.package_button.disabled = True
            self.remove_button.disabled = True
            #enable buttons for various options on resulting bag
            self.package_gui = ui.HBox([self.download_button])
            with self.out:
                display.display(self.package_gui)
        else: #couldn't create the directory to make the bag, throw a error in a dialog box
            self.modal_dialog('Error','Error packaging files. Could not create folder in session directory!')
            
    #remove selected files from packaging manifest
    def remove_files(self,obj_ignored):
        for item in self.items:
            if item.value:
                self.file_selector.selected_files.remove(item.description)
        self.review_files(None)
            
    #prints files selected with checkboxes allowing some to be removed
    def review_files(self,change):
        self.items = [ui.Checkbox(value=False,description=val,disabled=False,layout=Layout(width='90%')) 
                 for val in self.file_selector.selected_files]
        self.file_reviewer = ui.VBox(self.items)
        self.reviewer_gui = ui.HBox([self.remove_button,self.package_button])
        with self.out:
            IPython.display.clear_output()
            display.display(self.file_reviewer)
            if len(self.items) > 0:
                display.display(self.reviewer_gui)
                self.remove_button.disabled = False
                self.package_button.disabled = False

